;
;	65xx opcode definitions
;
.equ	LDA_		= 0xA9		; Opcode for LDA #<immediate>
.equ	STA_		= 0x8D		; Opcode for STA <absolute>
.equ	JMP_		= 0x4C		; Opcode for JMP <absolute>
;
;
;
;
imlresume:
	ldi		line, 1
	ldi		zl, low(vbporch_d)
	ldi		zh, high(vbporch_d)
	sts		nextstate, zl
	sts		nextstate+1, zh
	ldi		temp, 1<<OCIE1B			; 
	sts		TIMSK1, temp			; Enable hline ISR
	ret


imlreset:
	lds		r0, a1l
	sbrc	r0, 0
	sbi		RESETPIN
	sbrs	r0, 0
	cbi		RESETPIN
	ret
;	
imloutput:
	sts		TIMSK1, zero			; Disable hline ISR
	cbi		PORTB, KEY
	lds		r0, a1l
	out		PORTA, r0
	sbi		PORTB, KEY
	rjmp	imlresume
;
imlstop:
	cbi		PORTD, PHI2
	sts		TCCR2B, zero
	sts		TCCR2A, zero
	sts		OCR2A, zero
	ret
;
imlclock:
	lds		temp, a1l		; 0 = 11MHz, 0x0A is ~1MHz, 0x64 is ~ 100kHz
	andi	temp, 0x7F		; This works only for values up to 0x7F
	sts		OCR2B, temp
	lsl		temp
	inc		temp
	sts		OCR2A, temp
	ldi		temp, (1<<WGM22) | (0<<CS22) | (0<<CS21) | (1<<CS20)
	sts		TCCR2B, temp
	ldi		temp, (0<<COM2A1) | (0<<COM2A0) | (1<<COM2B1) | (1<<COM2B0) | (1<<WGM21) | (1<<WGM20)
	sts		TCCR2A, temp
	ret
;
;	Support routine output data, and strobe PHI2 high for a
;	short time. sbi and cbi use both two cycles and the
;	output is updated at the end. Using two <nop> will
;	strobe PHI2 for 4 AVR cycles, in our case this is 160ns.
;
imlbyte:
	cbi		PORTB, KEY
	out		PORTA, temp
	sbrc	temp, 7
	sbi		PORTD, PHI2
	nop
	sbi		PORTB, KEY
	sbi		PORTD, PHI2
	nop
	nop
	cbi		PORTD, PHI2	; PHI2 low
	ret
;	
imlcycle:
	sts		TIMSK1, zero		; Disable hline ISR

	lds		temp, a1l			; get byte
	rcall	imlbyte
	rjmp	imlresume
;
imlstatus:
	rcall	seroutcrlf
	ldi		char, 'V'
	rcall	serout
	ldi		char, 'P'
	rcall	serout
	ldi		char, 'B'
	rcall	serout
	ldi		char, ':'
	rcall	serout
	ldi		char, '1'
	sbis	PINB, VPB
	ldi		char, '0'
	rcall	serout
;
	ldi		char, ','
	rcall	serout
	ldi		char, ' '
	rcall	serout
;
	ldi		char, 'I'
	rcall	serout
	ldi		char, 'M'
	rcall	serout
	ldi		char, 'L'
	rcall	serout
	ldi		char, ':'
	rcall	serout
	ldi		char, '1'
	sbis	IMLPIN
	ldi		char, '0'
	rcall	serout
;
	ldi		char, ','
	rcall	serout
	ldi		char, ' '
	rcall	serout
;
	ldi		char, 'P'
	rcall	serout
	ldi		char, 'H'
	rcall	serout
	ldi		char, 'I'
	rcall	serout
	ldi		char, '2'
	rcall	serout
	ldi		char, ':'
	rcall	serout
	ldi		char, '1'
	sbis	PORTD, PHI2
	ldi		char, '0'
	rcall	serout
;
	ldi		char, ','
	rcall	serout
	ldi		char, ' '
	rcall	serout
;
	ldi		char, 'R'
	rcall	serout
	ldi		char, 'E'
	rcall	serout
	ldi		char, 'S'
	rcall	serout
	ldi		char, ':'
	rcall	serout
	ldi		char, '1'
	sbic	RESETPIN
	ldi		char, '0'
	rcall	serout
	ret
;	
imltoggle:
	sbi		PIND, PHI2
	ret
;
imliml:
	sbi		PINB, IML
	ret
;
;	Load default ROM image
;
imldefault:
	push	xl
	push	xh

	ldi		zl, low(2*romimage)
	ldi		zh, high(2*romimage)
	lds		temp, a1l
	cpi		temp, 0x01
	brne	imldefault010
	ldi		zl, low(2*romimage1)
	ldi		zh, high(2*romimage1)
imldefault010:
	lpm		xl, Z+
	lpm		xh, Z+

	rcall	imlprepare

wloop:
;
	ldi		temp, 0x00			; The first time we need to provide
	rcall	imlbyte
;
	ldi		temp, 0x03			; a RESET vector, any address will do so
	rcall	imlbyte				; we use the same address as in the JMP.
;
	ldi		temp, LDA_			; Load <immediate>
	rcall	imlbyte
;
	lpm		temp, Z+			; next byte from ROM image
	rcall	imlbyte
;
	ldi		temp, STA_			; Store <absolute>
	rcall	imlbyte
;
	mov		temp, xl			; to the current pointer
	rcall	imlbyte
;
	mov		temp, xh			; of the ROM image destination
	rcall	imlbyte
;
	ldi		temp, 0xff			; the write cycle, the value is not
	rcall	imlbyte				; relevant
;
	ldi		temp, JMP_			; Jump to start of loop
	rcall	imlbyte				; 
								; 
	adiw	xh:xl, 1			; Next ROM destination address
	brne	wloop				; if not overflow to 0 then continue

	rcall	videostart
	pop		xh					; Ok we stop in the middle of the JMP
	pop		xl					; instruction but we don't care, because
	ret							; next is de-asserting IML and do a reset.


;
;
;
imlerase:
	rcall	imlprepare
	ldi		zh, 0x03
	ldi		zl, 0xFF
eloop:
;
	ldi		temp, 0x00			; The first time we need to provide
	rcall	imlbyte
;
	ldi		temp, 0x03			; a RESET vector, any address will do so
	rcall	imlbyte				; we use the same address as in the JMP.
;
	ldi		temp, LDA_			; Load <immediate>
	rcall	imlbyte
;
	ldi		temp, 0xff
	rcall	imlbyte
;
	ldi		temp, STA_			; Store <absolute>
	rcall	imlbyte
;
	mov		temp, zl			; to the current pointer
	rcall	imlbyte
;
	mov		temp, zh			; of the ROM image destination
	rcall	imlbyte
;
	ldi		temp, 0xff			; the write cycle, the value is not
	rcall	imlbyte				; relevant
;
	ldi		temp, JMP_			; Jump to start of loop
	rcall	imlbyte				; 
								; 
	sbiw	zh:zl, 1			; Next ROM destination address
	brpl	eloop				; if not overflow to 0 then continue

	rcall	videostart
	ret

	
imlprepare:
	rcall	videostop
	rcall	imlstop
;
	sbi		IMLPIN
	sbi		RESETPIN
;
;	After Reset is asserted (uplled low) the 65816 requires two
;	full PHI2 cycles to reset.
;
	sbi		PORTD, PHI2		; Note that consecutive sbi/cbi
	cbi		PORTD, PHI2		; create a clock requency of 4 AVR
	sbi		PORTD, PHI2		; cycles in our case 5.5MHz.
	cbi		PORTD, PHI2
	sbi		PORTD, PHI2
	cbi		PORTD, PHI2
	sbi		PORTD, PHI2
;
;	De-assert Reset when PHI2 is high so the 65816 will detect
;	the rising edge of reset at the next falling edge of PHI2
;
	cbi		RESETPIN
	cbi		PORTD, PHI2		; this for some reason does not
							; detect reset being de-asserted
;
;	It seems that the number of cycles after a reset is not well
;	and accurately documented in the W65C816 manual from WDC. How-
;	ever it is in line with the Synertek manual of the 6502
;
;		http://archive.6502.org/datasheets/synertek_hardware_manual.pdf
;
;	which states that after RESET is de-asserted (high) the 6502 will
;	delay 6 cycles and then fetch the program counter, so we wait
;	6 cycles before emitting the start address of IML.
;
	sbi		PORTD, PHI2
	cbi		PORTD, PHI2		; This detects that reset is de-asserted
	sbi		PORTD, PHI2
	cbi		PORTD, PHI2		; Cylce 1
	sbi		PORTD, PHI2
	cbi		PORTD, PHI2		; Cylce 2
	sbi		PORTD, PHI2
	cbi		PORTD, PHI2		; Cylce 3
	sbi		PORTD, PHI2
	cbi		PORTD, PHI2		; Cylce 4
	sbi		PORTD, PHI2
	cbi		PORTD, PHI2		; Cylce 5
	ret
;
