;--------------------------------------------------------------------------
;
;	Scancode to ASCII decoder Statemachine
;
;

;
;	Flag bits for PS/2 keyboard
;
.equ	MLSHIFT		= 0
.equ	MRSHIFT		= 1
.equ	MLALT		= 2
.equ	MRALT		= 3
.equ	MLCTRL		= 4
.equ	MRCTRL		= 5
.equ	MOA			= 6
.equ	MCA			= 7
;
.equ	MSCRLL		= 0			; Scroll lock
.equ	MNUML		= 1
.equ	MCAPSL		= 2
;

	.dseg
ps2st:	.byte	2

	.cseg


ps2init:
ps2gotkey:
	ldi		zl, low(ps2key)
	ldi		zh, high(ps2key)
	sts		ps2st, zl
	sts		ps2st+1, zh
	ret

	
ps2key:
;	sbrs	Flagnormal,MOA
;	cbi		OPENAPPLE
;	sbrc	Flagnormal,MOA
;	sbi		OPENAPPLE		; Open Apple connected via Keyboard Connector to Input0
;	sbrs	Flagnormal,MCA
;	cbi		CLOSEAPPLE
;	sbrc	Flagnormal,MCA
;	sbi		CLOSEAPPLE		; Closed Apple connected via Keyboard Connector to Input1
	cpi		Scancode, 0xF0
	breq	release
	cpi		Scancode, 0xE0
	breq	extend
	cpi		Scancode, 0xE1
	breq	breakkey

	rcall	setflag
	brcc	ps2init
	rcall	sendcode
	brcs	ps2init
;
;;;	<we have a valid key>
;
	out		GPIOR1, r0		; Write Keyboard ASCII value
	sbi		GPIOR0, KWAIT
	rjmp	ps2gotkey

release:
	ldi		zl, low(ps2release)
	ldi		zh, high(ps2release)
	sts		ps2st, zl
	sts		ps2st+1, zh
	ret
ps2release:
	rcall	clrflag
	rjmp	ps2gotkey

extend:
	ldi		zl, low(ps2extend)
	ldi		zh, high(ps2extend)
	sts		ps2st, zl
	sts		ps2st+1, zh
	ret
ps2extend:
	cpi		Scancode,0xF0		; if the next scancode is 0xF0
	breq	extrel				; then an extended key has been released
	rcall	setflage			; Else an extended key has been pressed
	brcc	ps2gotkey			; if CC then the scancode was already processed
	rcall	sendextn			; else check whether it is a valid extended keystroke
	brcs	ps2gotkey			; if CS no, not a usefull key, get and try next key
;
;;;	<we have a valid key>
;
	out		GPIOR1, r0		; Write Keyboard ASCII value
	sbi		GPIOR0, KWAIT
	rjmp	ps2gotkey
;
;	An extended key has been released
;
extrel:
	ldi		zl, low(ps2extrel)
	ldi		zh, high(ps2extrel)
	sts		ps2st, zl
	sts		ps2st+1, zh
	ret
ps2extrel:
	rcall	clrflage			; It might affect the flags
	rjmp	ps2gotkey			; Process next key

breakkey:
	ldi		zl, low(ps2breakkey)
	ldi		zh, high(ps2breakkey)
	sts		ps2st, zl
	sts		ps2st+1, zh
	ret
ps2breakkey:
	rjmp	ps2gotkey


;--------------------------------------------------------------------------
;
;	Send scancode, i.e. process a scancode that 
;	translates into a key to be sent.
;
sendcode:
	cpi		Scancode,0x80
	brlo	sendn010
	sec
	ret
sendn010:
	sbrc	Flagnormal,MRALT
	rjmp	sendgraf				; Send GRAF if MRALT is pressed
	sbrc	Flagnormal,MLALT
	rjmp	sendspec				; Send SPECIAL if MLALT is pressed 
	sbrc	Flagnormal,MLCTRL
	rjmp	sendctrl				; Send CTRL if MLCTRL is pressed
	sbrc	Flagnormal,MRCTRL
	rjmp	sendctrl				; Send CTRL if MRCTRL is pressed
	sbrc	Flagnormal,MLSHIFT
	rjmp	sendshift				; Send SHIFT if MLSHIFT is pressed
	sbrc	Flagnormal,MRSHIFT
	rjmp	sendshift				; Send SHIFT if MRSHIFT is pressed
	sbrc	Flagtoggle,MCAPSL
	rjmp	sendcaps				; Send CAPSL if MCAPSL is pressed
;
;	Scancode is translated with normal keyboard table (no shift, no control
;	no nothing else pressed)
;
	ldi		zl,low(2*kbdnorm)		; If no modifier is pressed use normal table
	ldi		zh,high(2*kbdnorm)
	add		zl,Scancode
	adc		zh,zero
	lpm	
	sbrc	r0,7					; high bit indicates invalid scancode
	rjmp	sendn090				; which does not translate to key
	clc
	ret
;
;	Scancode is translated with Caps Locked and no Shift Key
;
sendcaps:							; Only Caps Locked but no Shift key
	ldi		zl,low(2*kbdcaps)
	ldi		zh,high(2*kbdcaps)
	add		zl,Scancode
	adc		zh,zero
	lpm
	sbrc	r0,7
	rjmp	sendn090
	clc
	ret
;
;	Scancode is translated with either Shift Key pressed
;
sendshift:							; Either Shift key is pressed
	sbrc	Flagtoggle,MCAPSL		; Send INVT if MCAPSL is pressed
	rjmp	sendn040
	ldi		zl,low(2*kbdshft)
	ldi		zh,high(2*kbdshft)
	add		zl,Scancode
	adc		zh,zero
	lpm
	sbrc	r0,7
	rjmp	sendn090
	clc
	ret
;
;	Scancode is translated with either Shift Key and Caps Lock active
;
sendn040:
	ldi		zl,low(2*kbdinvt)
	ldi		zh,high(2*kbdinvt)
	add		zl,Scancode
	adc		zh,zero
	lpm
	sbrc	r0,7
	rjmp	sendn090
	clc
	ret
;
sendn090:
	sec
	ret
;
;	Special means MLALT is not currently implemented
;
sendspec:
	sec
	ret




;--------------------------------------------------------------------------
;
;	Treat Flags Scancodes of pressed keys
;	First check the scancodes for keys
;	that toggle a bit. Carry is cleared
;	if scancode could be processes else
;	carry is set (invalid or unrecognised
;	scancode)
;
setflag:
	cpi		Scancode,0x7E		; Scroll Lock
	brne	setf010
	ldi		zl,1<<MSCRLL
	eor		Flagtoggle,zl
	clc
	ret
setf010:
	cpi		Scancode,0x77		; Num Lock
	brne	setf020
	ldi		zl,1<<MNUML
	eor		Flagtoggle,zl
	clc
	ret
setf020:
	cpi		Scancode,0x58		; Caps Lock
	brne	setf030
	ldi		zl,1<<MCAPSL
	eor		Flagtoggle,zl
	clc
	ret
;
;	T = 1 for SET operation 
;
setf030:
	set
	rjmp	clrf010
;--------------------------------------------------------------------------
;
;	Treat Flags Scancodes of released keys
;	T = 0 for CLR operation
;
clrflag:
	clt
;
;	Now we check for scan codes that affect
;	flags, the T bit is set to 1 for make
;	and set to 0 for break codes
;
;
clrf010:
	cpi		Scancode,0x12		; Left Shift
	brne	clrf020
	bld		Flagnormal,MLSHIFT
	clc
	ret
clrf020:
	cpi		Scancode,0x59		; Right Shift
	brne	clrf030
	bld		Flagnormal,MRSHIFT
	clc
	ret
clrf030:
	cpi		Scancode,0x11		; Left Alt
	brne	clrf040
	bld		Flagnormal,MLALT
	clc
	ret
clrf040:
	cpi		Scancode,0x14		; Left Control
	brne	clrf050
	bld		Flagnormal,MLCTRL
	clc
	ret
clrf050:
	sec
	ret

;--------------------------------------------------------------------------
;
;	The same logic for make and break
;	of keys with extended scancodes
;	is applied here
;
;
;	T = 1 for pressed keys
;
setflage:
	set
	rjmp	clrf010e
;
;	T = 0 for released keys
;
clrflage:
	clt
clrf010e:
	cpi		Scancode,0x11		; Right Alt
	brne	clrf020e
	bld		Flagnormal,MRALT
	clc
	ret
clrf020e:
	cpi		Scancode,0x14		; Right Control
	brne	clrf030e
	bld		Flagnormal,MRCTRL
	clc
	ret
clrf030e:
	cpi		Scancode,0x1f		; Open Apple
	brne	clrf040e
	bld		Flagnormal,MOA
	clc
	ret
clrf040e:
	cpi		Scancode,0x2f		; Close Apple
	brne	clrf050e
	bld		Flagnormal,MCA
	clc
	ret
clrf050e:
	sec
	ret

;--------------------------------------------------------------------------




grafscnc:							; Table of valid scan code
	.db		0x1e,0x26				; @, #
	.db		0x55,0x54				; ~, [
	.db		0x5b,0x52				; ], {
	.db		0x5d,0x61				; }, \
	.db		0x3d,0x00				; |
grafcode:							; Table of corresponding key value
	.db	 	'@', '#'
	.db	 	'~', '['
	.db	 	']', '{'
	.db	 	'}', '\\'
	.db	 	'|', ' '

sendgraf:
	ldi		zl,low(2*grafscnc)
	ldi		zh,high(2*grafscnc)
	ldi		Counter,2*(grafcode-grafscnc)
sendg010:
	lpm		r0,Z+
	cp		Scancode,r0
	breq	sendg020
	dec		Counter
	brpl	sendg010
	sec
	ret
sendg020:
	adiw	Z,2*(grafcode-grafscnc)-1
	lpm
	clc
	ret	

;--------------------------------------------------------------------------
;
;	Send Extended Keyboard Scancodes values
;
extscnc:
	.db		0x69,0x6b
	.db		0x6c,0x72
	.db		0x74,0x75
	.db		0x4a,0x5a
	.db		0x71,0x80
extcode:
	.db		0x0e,0x08				; ^N, LEFT
	.db		0x02,0x0a				; ^B, DOWN
	.db		0x15,0x0b				; RIGHT,UP
	.db		 '/',0x0d				; /, RETURN
	.db		0x7f,0x00				; DEL
;
;
;
sendextn:
	ldi		zl,low(2*extscnc)
	ldi		zh,high(2*extscnc)
	ldi		Counter,2*(extcode-extscnc)
sende010:
	lpm		r0,Z+
	cp		Scancode,r0
	breq	sende020
	dec		Counter
	brpl	sende010
	sec
	ret
sende020:
	adiw	Z,2*(extcode-extscnc)-1
	lpm
	mov		zl,r0
	cpi		zl,0x7F					; Was it the DEL key?
	brne	sende050
	mov		zl,Flagnormal
	cbr		zl,(1<<MOA)|(1<<MCA)	; Ignore Open Apple
	cpi		zl,(1<<MLCTRL)|(1<<MLALT)
	brne	sende050				; only CTRL-ALT-DEL is honoured
	sbi		RESETPIN				; Setting pin as output activates

	lds		zl, OCR0A				; This is our 6502 Clock generator
	tst		zl						; And this is top so we need to burn
	breq	sende040				; at least 4 times cycles as OCR0A
sende030:
	nop
	nop
	dec		zl
	brne	sende030
sende040:
	nop
	nop
	nop
	nop
	cbi		RESETPIN				; set /RESET = Input = High
sende050:
	sec								; 
	ret


;--------------------------------------------------------------------------

sendctrl:
	ldi		zl, low(2*kbdctrl)
	ldi		zh, high(2*kbdctrl)
	add		zl, Scancode
	adc		zh, zero
	lpm		r0, Z
	clc
	sbrs	r0, 7
	ret
	sec
	ret
;--------------------------------------------------------------------------
;
;	All tables assume that the SCAN Codes are in the range 0x00...0x7F
;	this is true except for F7, therefore the Scan code for F7 (0x83) is
;	mapped to 0x02 (an unused Scan code)
;

kbdctrl:
;
;		0x00-0x0F
;
	.db	0x80,0x80,0x80,0x80		;	n.a.,	F9,		F7(*),	F5
	.db	0x80,0x80,0x80,0x80		;	F3,		F1,		F2,		F12
	.db	0x80,0x80,0x80,0x80		;	n.a.	F10,	F8,		F6
	.db	0x80,0x80,0x80,0x80		;	F4,		TAB,	,		n.a.
;
;		0x10-0x1F
;
	.db	0x80,0x80,0x80,0x80		;	n.a.	ALTL,	SHFL,	n.a.
	.db	0x80,0x11,0x80,0x80		;	CTRL,	q,		1,		n.a.
	.db	0x80,0x80,0x19,0x13		;	n.a.,	n.a.,	y,		s
	.db	0x01,0x17,0x80,0x80		;	a,		w,		2,		n.a.
;
;		0x20-0x2F
;
	.db	0x80,0x03,0x18,0x04		;	n.a.,	c,		x,		d
	.db	0x05,0x80,0x80,0x80		;	e,		4,		3,		n.a.
	.db	0x80,0x00,0x16,0x06		;	n.a.,	blnk,	v,		f
	.db	0x14,0x12,0x80,0x80		;	t,		r,		5,		n.a.
;
;		0x30-0x3F
;
	.db	0x80,0x0E,0x02,0x08		;	n.a.,	n,		b,		h
	.db	0x07,0x1A,0x80,0x80		;	g,		z,		6,		n.a.
	.db	0x80,0x80,0x0D,0x0A		;	n.a.,	n.a.,	m,		j
	.db	0x15,0x80,0x80,0x80		;	u,		7,		8,		n.a.
;
;		0x40-0x4F
;
	.db	0x80,0x80,0x0B,0x09		;	n.a.,	,,		k,		i
	.db	0x0F,0x80,0x80,0x80		;	o,		0,		9,		n.a.
	.db	0x80,0x80,0x1F,0x0C		;	n.a.,	.,		_,		l
	.db	0x80,0x10,0x80,0x80		;	n.a.,	p,		',		n.a.
;
;		0x50-0x5F
;
	.db	0x80,0x80,0x80,0x80		;	n.a.,	n.a.,	n.a.,	n.a.
	.db	0x1B,0x1E,0x80,0x80		;	[,		^,		n.a.,	n.a.
	.db	0x80,0x80,0x80,0x1D		;	n.a.,	n.a.,	<CR>,	]
	.db	0x80,0x80,0x80,0x80		;	n.a.,	$,		n.a.,	n.a.
;
;		0x60-0x6F
;
	.db	0x80,0x1C,0x80,0x80		;	n.a.,	<,		n.a.,	n.a.
	.db	0x80,0x80,0x80,0x80		;	n.a.,	n.a.,	BKSP,	n.a.
	.db	0x80,0x80,0x80,0x80		;	n.a.,	1,		n.a.,	4(NUM)
	.db	0x80,0x80,0x80,0x80		;	7(NUM),	n.a.,	n.a.,	n.a.(NUM)	
;
;		0x70-0x7F
;
	.db	0x80,0x80,0x80,0x80		;	0(NUM),	.(NUM),	2(NUM),	5(NUM)
	.db	0x80,0x80,0x80,0x80		;	6(NUM),	8(NUM),	ESC,	NUMLOCK
	.db	0x80,0x80,0x80,0x80		;	F11,	+(NUM),	3(NUM),	-(NUM)
	.db	0x80,0x80,0x80,0x80		;	*(NUM),	9(NUM),	SCRL,	none

kbdnorm:
	.db	0x80,0x80,0x80,0x80		;	n.a.,	F9,	F7,	F5
	.db	0x80,0x80,0x80,0x80		;	F3,	F1,	F2,	F12
	.db	0x80,0x80,0x80,0x80		;	n.a.	F10,	F8,	F6
	.db	0x80,0x09,0x80,0x80		;	F4,	TAB,	,	n.a.

	.db	0x80,0x80,0x80,0x80		;	n.a.	ALTL,	SHFL,	n.a.
	.db	0x80, 'q', '1',0x80		;	CTRL,	q,	1,	n.a.
	.db	0x80,0x80, 'y', 's'		;	n.a.,	n.a.,	y,	s
	.db	 'a', 'w', '2',0x80		;	a,	w,	2,	n.a.

	.db	0x80, 'c', 'x', 'd'		;	n.a.,	c,	x,	d
	.db	 'e', '4', '3',0x80		;	e,	4,	3,	n.a.
	.db	0x80, ' ', 'v', 'f'		;	n.a., blnk,	v,	f
	.db	 't', 'r', '5',0x80		;	t,	r,	5,	n.a.

	.db	0x80, 'n', 'b', 'h'		;	n.a.,	n,	b,	h
	.db	 'g', 'z', '6',0x80		;	g,	z,	6,	n.a.
	.db	0x80,0x80, 'm', 'j'		;	n.a.,	n.a.,	m,	j
	.db	 'u', '7', '8',0x80		;	u,	7,	8,	n.a.

	.db	0x80, ',', 'k', 'i'		;	n.a.,	,,	k,	i
	.db	 'o', '0', '9',0x80		;	o,	0,	9,	n.a.
	.db	0x80, '.', '-', 'l'		;	n.a.,	.,	-,	l
	.db	0x80, 'p', "'",0x80		;	n.a.,	p,	',	n.a.
	
	.db	0x80,0x80,0x80,0x80		;	n.a.,	n.a.,	n.a.,	n.a.
	.db	0x80, '^',0x80,0x80		;	n.a.,	^,	n.a.,	n.a.
	.db	0x80,0x80,0x0d, '!'		;	n.a.,	n.a.,	<CR>,	!
	.db	0x80, '$',0x80,0x80		;	n.a.,	$,	n.a.,	n.a.

	.db	0x80, '<',0x80,0x80		;	n.a.,	<,	n.a.,	n.a.
	.db	0x80,0x80,0x08,0x80		;	n.a.,	n.a.,	BKSP,	n.a.
	.db	0x80, '1',0x80, '4'		;	n.a.,	1,	n.a.,	4	NUMERIC KEYPAD
	.db	 '7',0x80,0x80,0x80		;	7,	n.a.,	n.a.,	n.a.	NUMERIC KEYPAD

	.db	 '0', '.', '2', '5'		;	0,	.,	2,	5	NUMERIC KEYPAD
	.db	 '6', '8',0x1b,0x80		;	6,	8,	ESC,	n.a.	NUMERIC KEYPAD
	.db	0x80, '+', '3', '-'		;	F11,	+,	3,	-	NUMERIC KEYPAD
	.db	 '*', '9',0x80,0x80		;	*,	9,	n.a.,	n.a.	NUMERIC KEYPAD

kbdshft:
	.db	0x80,0x80,0x80,0x80		;	n.a.,	F9,	F7,	F5
	.db	0x80,0x80,0x80,0x80		;	F3,	F1,	F2,	F12
	.db	0x80,0x80,0x80,0x80		;	n.a.	F10,	F8,	F6
	.db	0x80,0x09,0x80,0x80		;	F4,	TAB,	,	n.a.

	.db	0x80,0x80,0x80,0x80		;	n.a.	ALTL,	SHFL,	n.a.
	.db	0x80, 'Q', '+',0x80		;	CTRL,	q,	1,	n.a.
	.db	0x80,0x80, 'Y', 'S'		;	n.a.,	n.a.,	y,	s
	.db	 'A', 'W', '"',0x80		;	a,	w,	2,	n.a.

	.db	0x80, 'C', 'X', 'D'		;	n.a.,	c,	x,	d
	.db	 'E', '', '*',0x80		;	e,	4,	3,	n.a.
	.db	0x80, ' ', 'V', 'F'		;	n.a., blnk,	v,	f
	.db	 'T', 'R', '%',0x80		;	t,	r,	5,	n.a.

	.db	0x80, 'N', 'B', 'H'		;	n.a.,	n,	b,	h
	.db	 'G', 'Z', '&',0x80		;	g,	z,	6,	n.a.
	.db	0x80,0x80, 'M', 'J'		;	n.a.,	n.a.,	m,	j
	.db	 'U', '/', '(',0x80		;	u,	7,	8,	n.a.

	.db	0x80, ';', 'K', 'I'		;	n.a.,	,,	k,	i
	.db	 'O', '=', ')',0x80		;	o,	0,	9,	n.a.
	.db	0x80, ':', '_', 'L'		;	n.a.,	.,	-,	l
	.db	0x80, 'P', "?",0x80		;	n.a.,	p,	',	n.a.
	
	.db	0x80,0x80,0x80,0x80		;	n.a.,	n.a.,	n.a.,	n.a.
	.db	0x80, '`',0x80,0x80		;	n.a.,	^,	n.a.,	n.a.
	.db	0x80,0x80,0x0d, '!'		;	n.a.,	n.a.,	<CR>,	!
	.db	0x80, '',0x80,0x80		;	n.a.,	$,	n.a.,	n.a.

	.db	0x80, '>',0x80,0x80		;	n.a.,	<,	n.a.,	n.a.
	.db	0x80,0x80,0x08,0x80		;	n.a.,	n.a.,	BKSP,	n.a.
	.db	0x80, '1',0x80, '4'		;	n.a.,	1,	n.a.,	4	NUMERIC KEYPAD
	.db	 '7',0x80,0x80,0x80		;	7,	n.a.,	n.a.,	n.a.	NUMERIC KEYPAD

	.db	 '0', '.', '2', '5'		;	0,	.,	2,	5	NUMERIC KEYPAD
	.db	 '6', '8',0x1b,0x80		;	6,	8,	ESC,	n.a.	NUMERIC KEYPAD
	.db	0x80, '+', '3', '-'		;	F11,	+,	3,	-	NUMERIC KEYPAD
	.db	 '*', '9',0x80,0x80		;	*,	9,	n.a.,	n.a.	NUMERIC KEYPAD

kbdcaps:
	.db	0x80,0x80,0x80,0x80		;	n.a.,	F9,	F7,	F5
	.db	0x80,0x80,0x80,0x80		;	F3,	F1,	F2,	F12
	.db	0x80,0x80,0x80,0x80		;	n.a.	F10,	F8,	F6
	.db	0x80,0x09,0x80,0x80		;	F4,	TAB,	,	n.a.

	.db	0x80,0x80,0x80,0x80		;	n.a.	ALTL,	SHFL,	n.a.
	.db	0x80, 'Q', '1',0x80		;	CTRL,	q,	1,	n.a.
	.db	0x80,0x80, 'Y', 'S'		;	n.a.,	n.a.,	y,	s
	.db	 'A', 'W', '2',0x80		;	a,	w,	2,	n.a.

	.db	0x80, 'C', 'X', 'D'		;	n.a.,	c,	x,	d
	.db	 'E', '4', '3',0x80		;	e,	4,	3,	n.a.
	.db	0x80, ' ', 'V', 'F'		;	n.a., blnk,	v,	f
	.db	 'T', 'R', '5',0x80		;	t,	r,	5,	n.a.

	.db	0x80, 'N', 'B', 'H'		;	n.a.,	n,	b,	h
	.db	 'G', 'Z', '6',0x80		;	g,	z,	6,	n.a.
	.db	0x80,0x80, 'M', 'J'		;	n.a.,	n.a.,	m,	j
	.db	 'U', '7', '8',0x80		;	u,	7,	8,	n.a.

	.db	0x80, ',', 'K', 'I'		;	n.a.,	,,	k,	i
	.db	 'O', '0', '9',0x80		;	o,	0,	9,	n.a.
	.db	0x80, '.', '-', 'L'		;	n.a.,	.,	-,	l
	.db	0x80, 'P', "'",0x80		;	n.a.,	p,	',	n.a.
	
	.db	0x80,0x80,0x80,0x80		;	n.a.,	n.a.,	n.a.,	n.a.
	.db	0x80, '^',0x80,0x80		;	n.a.,	^,	n.a.,	n.a.
	.db	0x80,0x80,0x0d,0x80		;	n.a.,	n.a.,	<CR>,	n.a.
	.db	0x80, '$',0x80,0x80		;	n.a.,	$,	n.a.,	n.a.

	.db	0x80, '<',0x80,0x80		;	n.a.,	<,	n.a.,	n.a.
	.db	0x80,0x80,0x08,0x80		;	n.a.,	n.a.,	BKSP,	n.a.
	.db	0x80, '1',0x80, '4'		;	n.a.,	1,	n.a.,	4	NUMERIC KEYPAD
	.db	 '7',0x80,0x80,0x80		;	7,	n.a.,	n.a.,	n.a.	NUMERIC KEYPAD

	.db	 '0', '.', '2', '5'		;	0,	.,	2,	5	NUMERIC KEYPAD
	.db	 '6', '8',0x1b,0x80		;	6,	8,	ESC,	n.a.	NUMERIC KEYPAD
	.db	0x80, '+', '3', '-'		;	F11,	+,	3,	-	NUMERIC KEYPAD
	.db	 '*', '9',0x80,0x80		;	*,	9,	n.a.,	n.a.	NUMERIC KEYPAD

kbdinvt:
	.db	0x80,0x80,0x80,0x80		;	n.a.,	F9,	F7,	F5
	.db	0x80,0x80,0x80,0x80		;	F3,	F1,	F2,	F12
	.db	0x80,0x80,0x80,0x80		;	n.a.	F10,	F8,	F6
	.db	0x80,0x09,0x80,0x80		;	F4,	TAB,	,	n.a.

	.db	0x80,0x80,0x80,0x80		;	n.a.	ALTL,	SHFL,	n.a.
	.db	0x80, 'q', '+',0x80		;	CTRL,	q,	1,	n.a.
	.db	0x80,0x80, 'y', 's'		;	n.a.,	n.a.,	y,	s
	.db	 'a', 'w', '"',0x80		;	a,	w,	2,	n.a.

	.db	0x80, 'c', 'x', 'd'		;	n.a.,	c,	x,	d
	.db	 'e', '', '*',0x80		;	e,	4,	3,	n.a.
	.db	0x80, ' ', 'v', 'f'		;	n.a., blnk,	v,	f
	.db	 't', 'r', '%',0x80		;	t,	r,	5,	n.a.

	.db	0x80, 'n', 'b', 'h'		;	n.a.,	n,	b,	h
	.db	 'g', 'z', '&',0x80		;	g,	z,	6,	n.a.
	.db	0x80,0x80, 'm', 'j'		;	n.a.,	n.a.,	m,	j
	.db	 'u', '/', '(',0x80		;	u,	7,	8,	n.a.

	.db	0x80, ';', 'k', 'j'		;	n.a.,	,,	k,	i
	.db	 'o', '=', ')',0x80		;	o,	0,	9,	n.a.
	.db	0x80, ':', '_', 'l'		;	n.a.,	.,	-,	l
	.db	0x80, 'p', "?",0x80		;	n.a.,	p,	',	n.a.
	
	.db	0x80,0x80,0x80,0x80		;	n.a.,	n.a.,	n.a.,	n.a.
	.db	0x80, '`',0x80,0x80		;	n.a.,	^,	n.a.,	n.a.
	.db	0x80,0x80,0x0d,0x80		;	n.a.,	n.a.,	<CR>,	n.a.
	.db	0x80, '',0x80,0x80		;	n.a.,	$,	n.a.,	n.a.

	.db	0x80, '>',0x80,0x80		;	n.a.,	<,	n.a.,	n.a.
	.db	0x80,0x80,0x08,0x80		;	n.a.,	n.a.,	BKSP,	n.a.
	.db	0x80, '1',0x80, '4'		;	n.a.,	1,	n.a.,	4	NUMERIC KEYPAD
	.db	 '7',0x80,0x80,0x80		;	7,	n.a.,	n.a.,	n.a.	NUMERIC KEYPAD

	.db	 '0', '.', '2', '5'		;	0,	.,	2,	5	NUMERIC KEYPAD
	.db	 '6', '8',0x1b,0x80		;	6,	8,	ESC,	n.a.	NUMERIC KEYPAD
	.db	0x80, '+', '3', '-'		;	F11,	+,	3,	-	NUMERIC KEYPAD
	.db	 '*', '9',0x80,0x80		;	*,	9,	n.a.,	n.a.	NUMERIC KEYPAD
